@@ -1,8 +1,7 @@
 #' Tesseract Training Data
 #'
 #' Helper function to download training data from the official
-#' [tessdata](https://tesseract-ocr.github.io/tessdoc/Data-Files) repository. Only use this function on
-#' Windows and OS-X. On Linux, training data can be installed directly with
+#' [tessdata](https://tesseract-ocr.github.io/tessdoc/Data-Files) repository. On Linux, the fast training data can be installed directly with
 #' [yum](https://src.fedoraproject.org/rpms/tesseract) or
 #' [apt-get](https://packages.debian.org/search?suite=stable&section=all&arch=any&searchon=names&keywords=tesseract-ocr-).
 #'
@@ -23,31 +22,43 @@
 #' @family tesseract
 #' @param lang three letter code for language, see [tessdata](https://github.com/tesseract-ocr/tessdata) repository.
 #' @param datapath destination directory where to download store the file
+#' @param model either `fast` or `best` is currently supported. The latter downloads
+#' more accurate (but slower) trained models for Tesseract 4.0 or higher
 #' @param progress print progress while downloading
 #' @references [tesseract wiki: training data](https://tesseract-ocr.github.io/tessdoc/Data-Files)
 #' @examples \dontrun{
 #' if(is.na(match("fra", tesseract_info()$available)))
-#'   tesseract_download("fra")
+#'   tesseract_download("fra", model = 'best')
 #' french <- tesseract("fra")
 #' text <- ocr("https://jeroen.github.io/images/french_text.png", engine = french)
 #' cat(text)
 #' }
-tesseract_download <- function(lang, datapath = NULL, progress = interactive()){
+tesseract_download <- function(lang, datapath = NULL, model = c("fast", "best"), progress = interactive()) {
   stopifnot(is.character(lang))
+  model <- match.arg(model)
   if(!length(datapath)){
     warn_on_linux()
     datapath <- tesseract_info()$datapath
   }
   datapath <- normalizePath(datapath, mustWork = TRUE)
   version <- tesseract_version_major()
+
   if(version < 4){
     repo <- "tessdata"
     release <- "3.04.00"
   } else {
-    repo <- "tessdata_fast"
+    repo <- paste0("tessdata_", model)
     release <- "4.1.0"
   }
-  url <- sprintf('https://github.com/tesseract-ocr/%s/raw/%s/%s.traineddata', repo, release, lang)
+
+  url <- sprintf("https://github.com/tesseract-ocr/%s/raw/%s/%s.traineddata", repo, release, lang)
+
+  destfile <- file.path(datapath, basename(url))
+
+  if (file.exists(destfile)) {
+    message(paste("Training data already exists. Overwriting", destfile))
+  }
+
   req <- curl::curl_fetch_memory(url, curl::new_handle(
     progressfunction = progress_fun,
     noprogress = !isTRUE(progress)
@@ -56,7 +67,7 @@ tesseract_download <- function(lang, datapath = NULL, progress = interactive()){
     cat("\n")
   if(req$status_code != 200)
     stop("Download failed: HTTP ", req$status_code, call. = FALSE)
-  destfile <- file.path(datapath, basename(url))
+
   writeBin(req$content, destfile)
   return(destfile)
 }
